/* requires errno.h and stdarg.h */

#include "yaunc.h"

#include <errno.h>
#include <stdarg.h>
#include <syslog.h>
#include <stdio.h>

static void	err_doit(int, const char *, va_list);
static void	log_it(int, const char *, va_list);

/* char	*pname = NULL;	*/	/* caller can set this from argv[0] */

/* Nonfatal error related to a system call.
 * Print a message and return. */

void
err_ret(const char *fmt, ...)
{
	va_list		ap;

	va_start(ap, fmt);
	err_doit(1, fmt, ap);
	va_end(ap);
	return;
}

/* Fatal error related to a system call.
 * Print a message and terminate. */

void
err_sys(const char *fmt, ...)
{
	va_list		ap;

	va_start(ap, fmt);
	err_doit(1, fmt, ap);
	va_end(ap);
	exit(1);
}

/* Fatal error related to a system call.
 * Print a message, dump core, and terminate. */

void
err_dump(const char *fmt, ...)
{
	va_list		ap;

	va_start(ap, fmt);
	err_doit(1, fmt, ap);
	va_end(ap);
	abort();		/* dump core and terminate */
	exit(1);		/* shouldn't get here */
}

/* Nonfatal error unrelated to a system call.
 * Print a message and return. */

void
err_msg(const char *fmt, ...)
{
	va_list		ap;

	va_start(ap, fmt);
	err_doit(0, fmt, ap);
	va_end(ap);
	return;
}

/* Fatal error unrelated to a system call.
 * Print a message and terminate. */

void
err_quit(const char *fmt, ...)
{
	va_list		ap;

	va_start(ap, fmt);
	err_doit(0, fmt, ap);
	va_end(ap);
	exit(1);
}

/* same error stuff as above, except send data to syslog 
   (if we're in daemon mode, that is)
*/
void
err_log_ret(const char *fmt, ...)
{
	va_list 		ap;

	va_start(ap, fmt);
	if (GetDaemonFlag() != 0)
	   log_it(1, fmt, ap);
	else
	   err_doit(1, fmt, ap);
	va_end(ap);
	return;
}	

void
err_log_msg(const char *fmt, ...)
{
   va_list     ap;

   va_start(ap, fmt);
   if (GetDaemonFlag() != 0)
      log_it(0, fmt, ap);
   else
      err_doit(0, fmt, ap);
   va_end(ap);
   return;
} 

void
err_log_sys(const char *fmt, ...)
{
	va_list 		ap;

	va_start(ap, fmt);
	if (GetDaemonFlag() != 0)
	   log_it(1, fmt, ap);
	else
	   err_doit(1, fmt, ap);
	va_end(ap);
	exit(1);
}	

void
err_log_dump(const char *fmt, ...)
{
	va_list		ap;

	va_start(ap, fmt);
	if (GetDaemonFlag() != 0)
  	   log_it(1, fmt, ap);
	else
	   err_doit(1, fmt, ap);
	va_end(ap);
	abort();
	exit(1);
}	

/* Print a message and return to caller.
 * Caller specifies "errnoflag". */

static void
err_doit(int errnoflag, const char *fmt, va_list ap)
{
	int		errno_save;
	char	buf[MAXLINE];

	errno_save = errno;		/* value caller might want printed */
	vsprintf(buf, fmt, ap);
	if (errnoflag)
		sprintf(buf+strlen(buf), ": %s", strerror(errno_save));
	strcat(buf, "\n");
	fflush(stdout);		/* in case stdout and stderr are the same */
	fputs(buf, stderr);
	fflush(NULL);		/* flushes all stdio output streams */
	return;
}

static void
log_it(int errnoflag, const char *fmt, va_list ap)
{
	int		errno_save;
	char	buf[MAXLINE];

	errno_save = errno;		/* value caller might want printed */
	openlog(NAME, LOG_OPTION, LOG_FACILITY);
	vsprintf(buf, fmt, ap);
	if (errnoflag)
		sprintf(buf+strlen(buf), ": %s", strerror(errno_save));
	strcat(buf, "\n");
	/* write stuff to the syslog */
	syslog(LOG_PRIORITY, buf);
	closelog();
	return;
}
